@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: drv_sgi.c,v 1.1.1.1 2004/06/01 12:16:17 raph Exp $
+  $Id$
 
   Driver for output on SGI audio system (needs libaudio from the dmedia
   package).
@@ -69,18 +69,18 @@ static	int sgi_fragsize=DEFAULT_SGI_FRAGSIZE;
 static	int sgi_bufsize=DEFAULT_SGI_BUFSIZE;
 static	SBYTE *audiobuffer=NULL;
 
-static void SGI_CommandLine(CHAR *cmdline)
+static void SGI_CommandLine(const CHAR *cmdline)
 {
 	CHAR *ptr;
 
 	if ((ptr=MD_GetAtom("fragsize",cmdline,0))) {
 		sgi_fragsize=atol(ptr);
-		free(ptr);
+		MikMod_free(ptr);
 	} else sgi_fragsize=DEFAULT_SGI_FRAGSIZE;
 
 	if ((ptr=MD_GetAtom("bufsize",cmdline,0))) {
 		sgi_bufsize=atol(ptr);
-		free(ptr);
+		MikMod_free(ptr);
 	} else sgi_bufsize=DEFAULT_SGI_BUFSIZE;
 }
 
@@ -167,7 +167,7 @@ static int SGI_Init(void)
 		return 1;
 	}
 
-	if(!(audiobuffer=(SBYTE*)_mm_malloc(sgi_fragsize))) return 1;
+	if(!(audiobuffer=(SBYTE*)MikMod_malloc(sgi_fragsize))) return 1;
 
 	return VC_Init();
 }
@@ -175,7 +175,8 @@ static int SGI_Init(void)
 static void SGI_Exit(void)
 {
 	VC_Exit();
-	_mm_free(audiobuffer);
+	MikMod_free(audiobuffer);
+	audiobuffer=NULL;
 }
 
 static void SGI_Update(void)
@@ -190,7 +191,8 @@ MIKMODAPI MDRIVER drv_sgi={
 	"SGI Audio System driver v0.5",
 	0,255,
 	"sgi",
-
+	"fragsize:r:0,99999,20000:Sound buffer fragment size\n"
+        "bufsize:r:0,199999,40000:Sound buffer total size\n",
 	SGI_CommandLine,
 	SGI_IsThere,
 	VC_SampleLoad,
