@@ -6,12 +6,12 @@
 	it under the terms of the GNU Library General Public License as
 	published by the Free Software Foundation; either version 2 of
 	the License, or (at your option) any later version.
- 
+
 	This program is distributed in the hope that it will be useful,
 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 	GNU Library General Public License for more details.
- 
+
 	You should have received a copy of the GNU Library General Public
 	License along with this library; if not, write to the Free Software
 	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
@@ -50,6 +50,8 @@ extern long int random(void);
 /* The currently playing module */
 MODULE *pf = NULL;
 
+#define NUMVOICES(mod) (md_sngchn < (mod)->numvoices ? md_sngchn : (mod)->numvoices)
+
 #define	HIGH_OCTAVE		2	/* number of above-range octaves */
 
 static	UWORD oldperiods[OCTAVE*2]={
@@ -225,12 +227,12 @@ static	SBYTE PanbrelloTable[256]={
 };
 
 /* returns a random value between 0 and ceil-1, ceil must be a power of two */
-static int getrandom(int ceil)
+static int getrandom(int ceilval)
 {
 #ifdef HAVE_SRANDOM
-	return random()&(ceil-1);
+	return random()&(ceilval-1);
 #else
-	return (rand()*ceil)/(RAND_MAX+1.0);
+	return (rand()*ceilval)/(RAND_MAX+1.0);
 #endif
 }
 
@@ -246,14 +248,14 @@ static int MP_FindEmptyChannel(MODULE *mod)
 	MP_VOICE *a;
 	ULONG t,k,tvol,pp;
 
-	for (t=0;t<md_sngchn;t++)
+	for (t=0;t<NUMVOICES(mod);t++)
 		if (((mod->voice[t].main.kick==KICK_ABSENT)||
 			 (mod->voice[t].main.kick==KICK_ENV))&&
 		   Voice_Stopped_internal(t))
 			return t;
 
 	tvol=0xffffffUL;t=-1;a=mod->voice;
-	for (k=0;k<md_sngchn;k++,a++) {
+	for (k=0;k<NUMVOICES(mod);k++,a++) {
 		/* allow us to take over a nonexisting sample */
 		if (!a->main.s)
 			return k;
@@ -325,9 +327,9 @@ static UWORD GetPeriod(UWORD flags, UWORD note, ULONG speed)
 {
 	if (flags & UF_XMPERIODS) {
 		if (flags & UF_LINEAR)
-				return getlinearperiod(note, speed);
+			return getlinearperiod(note, speed);
 		else
-				return getlogperiod(note, speed);
+			return getlogperiod(note, speed);
 	} else
 		return getoldperiod(note, speed);
 }
@@ -361,7 +363,8 @@ static SWORD StartEnvelope(ENVPR *t,UBYTE flg,UBYTE pts,UBYTE susbeg,UBYTE susen
 
 	/* Imago Orpheus sometimes stores an extra initial point in the envelope */
 	if ((t->pts>=2)&&(t->env[0].pos==t->env[1].pos)) {
-		t->a++;t->b++;
+		t->a++;
+		t->b++;
 	}
 
 	/* Fit in the envelope, still */
@@ -607,10 +610,10 @@ static void DoToneSlide(UWORD tick, MP_CONTROL *a)
 			/* ...make tmpperiod equal tperiod */
 			a->tmpperiod=a->main.period=a->wantedperiod;
 		else if (dist>0) {
-			a->tmpperiod-=a->portspeed;	
+			a->tmpperiod-=a->portspeed;
 			a->main.period-=a->portspeed; /* dist>0, slide up */
 		} else {
-			a->tmpperiod+=a->portspeed;	
+			a->tmpperiod+=a->portspeed;
 			a->main.period+=a->portspeed; /* dist<0, slide down */
 		}
 	} else
@@ -827,8 +830,8 @@ static int DoPTEffectB(UWORD tick, UWORD flags, MP_CONTROL *a, MODULE *mod, SWOR
 	if (!mod->loop && !mod->patbrk &&
 	    (dat < mod->sngpos ||
 		 (mod->sngpos == (mod->numpos - 1) && !mod->patbrk) ||
-	     (dat == mod->sngpos && (flags & UF_NOWRAP))
-		)) {
+	     (dat == mod->sngpos && (flags & UF_NOWRAP)) ) )
+	{
 		/* if we don't loop, better not to skip the end of the
 		   pattern, after all... so:
 		mod->patbrk=0; */
@@ -849,11 +852,11 @@ static int DoPTEffectC(UWORD tick, UWORD flags, MP_CONTROL *a, MODULE *mod, SWOR
 {
 	UBYTE dat;
 
-			dat=UniGetByte();
-			if (tick) return 0;
-			if (dat==(UBYTE)-1) a->anote=dat=0; /* note cut */
-			else if (dat>64) dat=64;
-			a->tmpvolume=dat;
+	dat=UniGetByte();
+	if (tick) return 0;
+	if (dat==(UBYTE)-1) a->anote=dat=0; /* note cut */
+	else if (dat>64) dat=64;
+	a->tmpvolume=dat;
 
 	return 0;
 }
@@ -862,26 +865,26 @@ static int DoPTEffectD(UWORD tick, UWORD flags, MP_CONTROL *a, MODULE *mod, SWOR
 {
 	UBYTE dat;
 
-			dat=UniGetByte();
-			if ((tick)||(mod->patdly2)) return 0;
-			if ((mod->positions[mod->sngpos]!=LAST_PATTERN)&&
-			   (dat>mod->pattrows[mod->positions[mod->sngpos]]))
-				dat=mod->pattrows[mod->positions[mod->sngpos]];
-			mod->patbrk=dat;
-			if (!mod->posjmp) {
-				/* don't ask me to explain this code - it makes
-				   backwards.s3m and children.xm (heretic's version) play
-				   correctly, among others. Take that for granted, or write
-				   the page of comments yourself... you might need some
-				   aspirin - Miod */
-				if ((mod->sngpos==mod->numpos-1)&&(dat)&&((mod->loop)||
-				               (mod->positions[mod->sngpos]==(mod->numpat-1)
-								&& !(flags&UF_NOWRAP)))) {
-					mod->sngpos=0;
-					mod->posjmp=2;
-				} else
-					mod->posjmp=3;
-			}
+	dat=UniGetByte();
+	if ((tick)||(mod->patdly2)) return 0;
+	if ((mod->positions[mod->sngpos]!=LAST_PATTERN)&&
+	    (dat>mod->pattrows[mod->positions[mod->sngpos]])) {
+		dat=mod->pattrows[mod->positions[mod->sngpos]];
+	}
+	mod->patbrk=dat;
+	if (!mod->posjmp) {
+		/* don't ask me to explain this code - it makes
+		   backwards.s3m and children.xm (heretic's version) play
+		   correctly, among others. Take that for granted, or write
+		   the page of comments yourself... you might need some
+		   aspirin - Miod */
+		if ((mod->sngpos==mod->numpos-1)&&(dat)&&
+		    ((mod->loop) || (mod->positions[mod->sngpos]==(mod->numpat-1) && !(flags&UF_NOWRAP)))) {
+			mod->sngpos=0;
+			mod->posjmp=2;
+		} else
+			mod->posjmp=3;
+	}
 
 	return 0;
 }
@@ -1027,7 +1030,7 @@ static int DoPTEffectF(UWORD tick, UWORD flags, MP_CONTROL *a, MODULE *mod, SWOR
 	if (tick||mod->patdly2) return 0;
 	if (mod->extspd&&(dat>=mod->bpmlimit))
 		mod->bpm=dat;
-	else 
+	else
 	  if (dat) {
 		mod->sngspd=(dat>=mod->bpmlimit)?mod->bpmlimit-1:dat;
 		mod->vbtick=0;
@@ -1392,7 +1395,7 @@ static int DoXMEffectA(UWORD tick, UWORD flags, MP_CONTROL *a, MODULE *mod, SWOR
 		a->s3mvolslide = inf;
 	else
 		inf = a->s3mvolslide;
-	
+
 	if (tick) {
 		lo=inf&0xf;
 		hi=inf>>4;
@@ -1604,7 +1607,7 @@ static void DoITToneSlide(UWORD tick, MP_CONTROL *a, UBYTE dat)
 	/* if we don't come from another note, ignore the slide and play the note
 	   as is */
 	if (!a->oldnote || !a->main.period)
-			return;
+		return;
 
 	if ((!tick)&&(a->newsamp)){
 		a->main.kick=KICK_NOTE;
@@ -1619,15 +1622,15 @@ static void DoITToneSlide(UWORD tick, MP_CONTROL *a, UBYTE dat)
 		   difference between those two values */
 		dist=a->main.period-a->wantedperiod;
 
-	    /* if they are equal or if portamentospeed is too big... */
+		/* if they are equal or if portamentospeed is too big... */
 		if ((!dist)||((a->portspeed<<2)>abs(dist)))
 			/* ... make tmpperiod equal tperiod */
 			a->tmpperiod=a->main.period=a->wantedperiod;
 		else
-		  if (dist>0) {	
+		  if (dist>0) {
 			a->tmpperiod-=a->portspeed<<2;
 			a->main.period-=a->portspeed<<2; /* dist>0 slide up */
-		} else {				
+		} else {
 			a->tmpperiod+=a->portspeed<<2;
 			a->main.period+=a->portspeed<<2; /* dist<0 slide down */
 		}
@@ -1653,7 +1656,7 @@ static void DoITVibrato(UWORD tick, MP_CONTROL *a, UBYTE dat)
 		if (dat&0xf0) a->vibspd=(dat&0xf0)>>2;
 	}
 	if (!a->main.period)
-			return;
+		return;
 
 	q=(a->vibpos>>2)&0x1f;
 
@@ -1743,7 +1746,7 @@ static int DoITEffectN(UWORD tick, UWORD flags, MP_CONTROL *a, MODULE *mod, SWOR
 	lo=inf&0xf;
 	hi=inf>>4;
 
-	if (!hi) 
+	if (!hi)
 		a->main.chanvol-=lo;
 	else
 	  if (!lo) {
@@ -1806,7 +1809,7 @@ static int DoITEffectT(UWORD tick, UWORD flags, MP_CONTROL *a, MODULE *mod, SWOR
 	UBYTE tempo;
 	SWORD temp;
 
-   	tempo = UniGetByte();
+	tempo = UniGetByte();
 
 	if (mod->patdly2)
 		return 0;
@@ -1967,13 +1970,13 @@ static int DoITEffectS0(UWORD tick, UWORD flags, MP_CONTROL *a, MODULE *mod, SWO
 	switch (c) {
 	case SS_GLISSANDO: /* S1x set glissando voice */
 		DoEEffects(tick, flags, a, mod, channel, 0x30|inf);
-		break;              
+		break;
 	case SS_FINETUNE: /* S2x set finetune */
 		DoEEffects(tick, flags, a, mod, channel, 0x50|inf);
 		break;
 	case SS_VIBWAVE: /* S3x set vibrato waveform */
 		DoEEffects(tick, flags, a, mod, channel, 0x40|inf);
-		break;   
+		break;
 	case SS_TREMWAVE: /* S4x set tremolo waveform */
 		DoEEffects(tick, flags, a, mod, channel, 0x70|inf);
 		break;
@@ -1992,7 +1995,7 @@ static int DoITEffectS0(UWORD tick, UWORD flags, MP_CONTROL *a, MODULE *mod, SWO
 	case SS_SURROUND: /* S9x set surround sound */
 		if (mod->panflag)
 			a->main.panning = mod->panning[channel] = PAN_SURROUND;
-		break;    
+		break;
 	case SS_HIOFFSET: /* SAy set high order sample offset yxx00h */
 		if (!tick) {
 			a->hioffset=inf<<16;
@@ -2030,9 +2033,9 @@ static int DoITEffectS0(UWORD tick, UWORD flags, MP_CONTROL *a, MODULE *mod, SWO
 static int DoVolEffects(UWORD tick, UWORD flags, MP_CONTROL *a, MODULE *mod, SWORD channel)
 {
 	UBYTE c, inf;
-	
-	c = UniGetByte(); 
-	inf = UniGetByte(); 
+
+	c = UniGetByte();
+	inf = UniGetByte();
 
 	if ((!c)&&(!inf)) {
 		c=a->voleffect;
@@ -2156,68 +2159,68 @@ static int DoNothing(UWORD tick, UWORD flags, MP_CONTROL *a, MODULE *mod, SWORD
 typedef int (*effect_func) (UWORD, UWORD, MP_CONTROL *, MODULE *, SWORD);
 
 static effect_func effects[UNI_LAST] = {
-		DoNothing,		/* 0 */
-		DoNothing,		/* UNI_NOTE */
-		DoNothing,		/* UNI_INSTRUMENT */
-		DoPTEffect0,	/* UNI_PTEFFECT0 */
-		DoPTEffect1,	/* UNI_PTEFFECT1 */
-		DoPTEffect2,	/* UNI_PTEFFECT2 */
-		DoPTEffect3,	/* UNI_PTEFFECT3 */
-		DoPTEffect4,	/* UNI_PTEFFECT4 */
-		DoPTEffect5,	/* UNI_PTEFFECT5 */
-		DoPTEffect6,	/* UNI_PTEFFECT6 */
-		DoPTEffect7,	/* UNI_PTEFFECT7 */
-		DoPTEffect8,	/* UNI_PTEFFECT8 */
-		DoPTEffect9,	/* UNI_PTEFFECT9 */
-		DoPTEffectA,	/* UNI_PTEFFECTA */
-		DoPTEffectB,	/* UNI_PTEFFECTB */
-		DoPTEffectC,	/* UNI_PTEFFECTC */
-		DoPTEffectD,	/* UNI_PTEFFECTD */
-		DoPTEffectE,	/* UNI_PTEFFECTE */
-		DoPTEffectF,	/* UNI_PTEFFECTF */
-		DoS3MEffectA,	/* UNI_S3MEFFECTA */
-		DoS3MEffectD,	/* UNI_S3MEFFECTD */
-		DoS3MEffectE,	/* UNI_S3MEFFECTE */
-		DoS3MEffectF,	/* UNI_S3MEFFECTF */
-		DoS3MEffectI,	/* UNI_S3MEFFECTI */
-		DoS3MEffectQ,	/* UNI_S3MEFFECTQ */
-		DoS3MEffectR,	/* UNI_S3MEFFECTR */
-		DoS3MEffectT,	/* UNI_S3MEFFECTT */
-		DoS3MEffectU,	/* UNI_S3MEFFECTU */
-		DoKeyOff,		/* UNI_KEYOFF */
-		DoKeyFade,		/* UNI_KEYFADE */
-		DoVolEffects,	/* UNI_VOLEFFECTS */
-		DoPTEffect4,	/* UNI_XMEFFECT4 */
-		DoXMEffect6,	/* UNI_XMEFFECT6 */
-		DoXMEffectA,	/* UNI_XMEFFECTA */
-		DoXMEffectE1,	/* UNI_XMEFFECTE1 */
-		DoXMEffectE2,	/* UNI_XMEFFECTE2 */
-		DoXMEffectEA,	/* UNI_XMEFFECTEA */
-		DoXMEffectEB,	/* UNI_XMEFFECTEB */
-		DoXMEffectG,	/* UNI_XMEFFECTG */
-		DoXMEffectH,	/* UNI_XMEFFECTH */
-		DoXMEffectL,	/* UNI_XMEFFECTL */
-		DoXMEffectP,	/* UNI_XMEFFECTP */
-		DoXMEffectX1,	/* UNI_XMEFFECTX1 */
-		DoXMEffectX2,	/* UNI_XMEFFECTX2 */
-		DoITEffectG,	/* UNI_ITEFFECTG */
-		DoITEffectH,	/* UNI_ITEFFECTH */
-		DoITEffectI,	/* UNI_ITEFFECTI */
-		DoITEffectM,	/* UNI_ITEFFECTM */
-		DoITEffectN,	/* UNI_ITEFFECTN */
-		DoITEffectP,	/* UNI_ITEFFECTP */
-		DoITEffectT,	/* UNI_ITEFFECTT */
-		DoITEffectU,	/* UNI_ITEFFECTU */
-		DoITEffectW,	/* UNI_ITEFFECTW */
-		DoITEffectY,	/* UNI_ITEFFECTY */
-		DoNothing,		/* UNI_ITEFFECTZ */
-		DoITEffectS0,	/* UNI_ITEFFECTS0 */
-		DoULTEffect9,	/* UNI_ULTEFFECT9 */
-		DoMEDSpeed,		/* UNI_MEDSPEED */
-		DoMEDEffectF1,	/* UNI_MEDEFFECTF1 */
-		DoMEDEffectF2,	/* UNI_MEDEFFECTF2 */
-		DoMEDEffectF3,	/* UNI_MEDEFFECTF3 */
-		DoOktArp,		/* UNI_OKTARP */
+	DoNothing,		/* 0 */
+	DoNothing,		/* UNI_NOTE */
+	DoNothing,		/* UNI_INSTRUMENT */
+	DoPTEffect0,	/* UNI_PTEFFECT0 */
+	DoPTEffect1,	/* UNI_PTEFFECT1 */
+	DoPTEffect2,	/* UNI_PTEFFECT2 */
+	DoPTEffect3,	/* UNI_PTEFFECT3 */
+	DoPTEffect4,	/* UNI_PTEFFECT4 */
+	DoPTEffect5,	/* UNI_PTEFFECT5 */
+	DoPTEffect6,	/* UNI_PTEFFECT6 */
+	DoPTEffect7,	/* UNI_PTEFFECT7 */
+	DoPTEffect8,	/* UNI_PTEFFECT8 */
+	DoPTEffect9,	/* UNI_PTEFFECT9 */
+	DoPTEffectA,	/* UNI_PTEFFECTA */
+	DoPTEffectB,	/* UNI_PTEFFECTB */
+	DoPTEffectC,	/* UNI_PTEFFECTC */
+	DoPTEffectD,	/* UNI_PTEFFECTD */
+	DoPTEffectE,	/* UNI_PTEFFECTE */
+	DoPTEffectF,	/* UNI_PTEFFECTF */
+	DoS3MEffectA,	/* UNI_S3MEFFECTA */
+	DoS3MEffectD,	/* UNI_S3MEFFECTD */
+	DoS3MEffectE,	/* UNI_S3MEFFECTE */
+	DoS3MEffectF,	/* UNI_S3MEFFECTF */
+	DoS3MEffectI,	/* UNI_S3MEFFECTI */
+	DoS3MEffectQ,	/* UNI_S3MEFFECTQ */
+	DoS3MEffectR,	/* UNI_S3MEFFECTR */
+	DoS3MEffectT,	/* UNI_S3MEFFECTT */
+	DoS3MEffectU,	/* UNI_S3MEFFECTU */
+	DoKeyOff,	/* UNI_KEYOFF */
+	DoKeyFade,	/* UNI_KEYFADE */
+	DoVolEffects,	/* UNI_VOLEFFECTS */
+	DoPTEffect4,	/* UNI_XMEFFECT4 */
+	DoXMEffect6,	/* UNI_XMEFFECT6 */
+	DoXMEffectA,	/* UNI_XMEFFECTA */
+	DoXMEffectE1,	/* UNI_XMEFFECTE1 */
+	DoXMEffectE2,	/* UNI_XMEFFECTE2 */
+	DoXMEffectEA,	/* UNI_XMEFFECTEA */
+	DoXMEffectEB,	/* UNI_XMEFFECTEB */
+	DoXMEffectG,	/* UNI_XMEFFECTG */
+	DoXMEffectH,	/* UNI_XMEFFECTH */
+	DoXMEffectL,	/* UNI_XMEFFECTL */
+	DoXMEffectP,	/* UNI_XMEFFECTP */
+	DoXMEffectX1,	/* UNI_XMEFFECTX1 */
+	DoXMEffectX2,	/* UNI_XMEFFECTX2 */
+	DoITEffectG,	/* UNI_ITEFFECTG */
+	DoITEffectH,	/* UNI_ITEFFECTH */
+	DoITEffectI,	/* UNI_ITEFFECTI */
+	DoITEffectM,	/* UNI_ITEFFECTM */
+	DoITEffectN,	/* UNI_ITEFFECTN */
+	DoITEffectP,	/* UNI_ITEFFECTP */
+	DoITEffectT,	/* UNI_ITEFFECTT */
+	DoITEffectU,	/* UNI_ITEFFECTU */
+	DoITEffectW,	/* UNI_ITEFFECTW */
+	DoITEffectY,	/* UNI_ITEFFECTY */
+	DoNothing,	/* UNI_ITEFFECTZ */
+	DoITEffectS0,	/* UNI_ITEFFECTS0 */
+	DoULTEffect9,	/* UNI_ULTEFFECT9 */
+	DoMEDSpeed,	/* UNI_MEDSPEED */
+	DoMEDEffectF1,	/* UNI_MEDEFFECTF1 */
+	DoMEDEffectF2,	/* UNI_MEDEFFECTF2 */
+	DoMEDEffectF3,	/* UNI_MEDEFFECTF3 */
+	DoOktArp,	/* UNI_OKTARP */
 };
 
 static int pt_playeffects(MODULE *mod, SWORD channel, MP_CONTROL *a)
@@ -2229,9 +2232,13 @@ static int pt_playeffects(MODULE *mod, SWORD channel, MP_CONTROL *a)
 	effect_func f;
 
 	while((c=UniGetByte())) {
+#if 0 /* this doesn't normally happen unless things go fubar elsewhere */
+		if (c >= UNI_LAST)
+		    fprintf(stderr,"fubar'ed opcode %u\n",c);
+#endif
 		f = effects[c];
 		if (f != DoNothing)
-				a->sliding = 0;
+		    a->sliding = 0;
 		explicitslides |= f(tick, flags, a, mod, channel);
 	}
 	return explicitslides;
@@ -2242,17 +2249,17 @@ static void DoNNAEffects(MODULE *mod, MP_CONTROL *a, UBYTE dat)
 	int t;
 	MP_VOICE *aout;
 
-	dat&=0xf; 
+	dat&=0xf;
 	aout=(a->slave)?a->slave:NULL;
 
 	switch (dat) {
 	case 0x0: /* past note cut */
-		for (t=0;t<md_sngchn;t++)
+		for (t=0;t<NUMVOICES(mod);t++)
 			if (mod->voice[t].master==a)
 				mod->voice[t].main.fadevol=0;
 		break;
 	case 0x1: /* past note off */
-		for (t=0;t<md_sngchn;t++)
+		for (t=0;t<NUMVOICES(mod);t++)
 			if (mod->voice[t].master==a) {
 				mod->voice[t].main.keyoff|=KEY_OFF;
 				if ((!(mod->voice[t].venv.flg & EF_ON))||
@@ -2261,7 +2268,7 @@ static void DoNNAEffects(MODULE *mod, MP_CONTROL *a, UBYTE dat)
 			}
 		break;
 	case 0x2: /* past note fade */
-		for (t=0;t<md_sngchn;t++)
+		for (t=0;t<NUMVOICES(mod);t++)
 			if (mod->voice[t].master==a)
 				mod->voice[t].main.keyoff|=KEY_FADE;
 		break;
@@ -2273,7 +2280,7 @@ static void DoNNAEffects(MODULE *mod, MP_CONTROL *a, UBYTE dat)
 		break;
 	case 0x5: /* set NNA note off */
 		a->main.nna=(a->main.nna&~NNA_MASK)|NNA_OFF;
-		break;   
+		break;
 	case 0x6: /* set NNA note fade */
 		a->main.nna=(a->main.nna&~NNA_MASK)|NNA_FADE;
 		break;
@@ -2288,7 +2295,7 @@ static void DoNNAEffects(MODULE *mod, MP_CONTROL *a, UBYTE dat)
 	case 0x9: /* disable panning envelope */
 		if (aout)
 			aout->main.panflg&=~EF_ON;
-		break;    
+		break;
 	case 0xa: /* enable panning envelope */
 		if (aout)
 			aout->main.panflg|=EF_ON;
@@ -2304,7 +2311,7 @@ static void DoNNAEffects(MODULE *mod, MP_CONTROL *a, UBYTE dat)
 	}
 }
 
-void pt_UpdateVoices(MODULE *mod, int max_volume)
+static void pt_UpdateVoices(MODULE *mod, int max_volume)
 {
 	SWORD envpan,envvol,envpit,channel;
 	UWORD playperiod;
@@ -2316,7 +2323,7 @@ void pt_UpdateVoices(MODULE *mod, int max_volume)
 	SAMPLE *s;
 
 	mod->totalchn=mod->realchn=0;
-	for (channel=0;channel<pf->numchn;channel++) {
+	for (channel=0;channel<NUMVOICES(mod);channel++) {
 		aout=&mod->voice[channel];
 		i=aout->main.i;
 		s=aout->main.s;
@@ -2507,7 +2514,7 @@ void pt_UpdateVoices(MODULE *mod, int max_volume)
 }
 
 /* Handles new notes or instruments */
-void pt_Notes(MODULE *mod)
+static void pt_Notes(MODULE *mod)
 {
 	SWORD channel;
 	MP_CONTROL *a;
@@ -2645,7 +2652,7 @@ void pt_Notes(MODULE *mod)
 }
 
 /* Handles effects */
-void pt_EffectsPass1(MODULE *mod)
+static void pt_EffectsPass1(MODULE *mod)
 {
 	SWORD channel;
 	MP_CONTROL *a;
@@ -2696,7 +2703,7 @@ void pt_EffectsPass1(MODULE *mod)
 }
 
 /* NNA management */
-void pt_NNA(MODULE *mod)
+static void pt_NNA(MODULE *mod)
 {
 	SWORD channel;
 	MP_CONTROL *a;
@@ -2737,7 +2744,7 @@ void pt_NNA(MODULE *mod)
 			if (a->dct!=DCT_OFF) {
 				int t;
 
-				for (t=0;t<md_sngchn;t++)
+				for (t=0;t<NUMVOICES(mod);t++)
 					if ((!Voice_Stopped_internal(t))&&
 					   (mod->voice[t].masterchn==channel)&&
 					   (a->main.sample==mod->voice[t].main.sample)) {
@@ -2777,7 +2784,7 @@ void pt_NNA(MODULE *mod)
 }
 
 /* Setup module and NNA voices */
-void pt_SetupVoices(MODULE *mod)
+static void pt_SetupVoices(MODULE *mod)
 {
 	SWORD channel;
 	MP_CONTROL *a;
@@ -2797,7 +2804,7 @@ void pt_SetupVoices(MODULE *mod)
 					if ((newchn=MP_FindEmptyChannel(mod))!=-1)
 						a->slave=&mod->voice[a->slavechn=newchn];
 				}
-			} else 
+			} else
 				a->slave=&mod->voice[a->slavechn=channel];
 
 			/* assign parts of MP_VOICE only done for a KICK_NOTE */
@@ -2818,7 +2825,7 @@ void pt_SetupVoices(MODULE *mod)
 }
 
 /* second effect pass */
-void pt_EffectsPass2(MODULE *mod)
+static void pt_EffectsPass2(MODULE *mod)
 {
 	SWORD channel;
 	MP_CONTROL *a;
@@ -2862,7 +2869,7 @@ void Player_HandleTick(void)
 	pf->sngremainder%=pf->bpm;
 
 	if (++pf->vbtick>=pf->sngspd) {
-		if (pf->pat_repcrazy) 
+		if (pf->pat_repcrazy)
 			pf->pat_repcrazy=0; /* play 2 times row 0 */
 		else
 			pf->patpos++;
@@ -2892,6 +2899,9 @@ void Player_HandleTick(void)
 				pf->control[channel].pat_reppos=-1;
 
 			pf->patbrk=pf->posjmp=0;
+
+			if (pf->sngpos<0) pf->sngpos=(SWORD)(pf->numpos-1);
+
 			/* handle the "---" (end of song) pattern since it can occur
 			   *inside* the module in some formats */
 			if ((pf->sngpos>=pf->numpos)||
@@ -2906,7 +2916,6 @@ void Player_HandleTick(void)
 					pf->bpm=pf->inittempo<32?32:pf->inittempo;
 				}
 			}
-			if (pf->sngpos<0) pf->sngpos=pf->numpos-1;
 		}
 
 		if (!pf->patdly2)
@@ -2939,7 +2948,7 @@ static void Player_Init_internal(MODULE* mod)
 		mod->control[t].main.chanvol=mod->chanvol[t];
 		mod->control[t].main.panning=mod->panning[t];
 	}
-	
+
 	mod->sngtime=0;
 	mod->sngremainder=0;
 
@@ -2963,7 +2972,7 @@ static void Player_Init_internal(MODULE* mod)
 	mod->patbrk=0;
 }
 
-BOOL Player_Init(MODULE* mod)
+int Player_Init(MODULE* mod)
 {
 	mod->extspd=1;
 	mod->panflag=1;
@@ -2979,6 +2988,11 @@ BOOL Player_Init(MODULE* mod)
 	if (!(mod->voice=(MP_VOICE*)MikMod_calloc(md_sngchn,sizeof(MP_VOICE))))
 		return 1;
 
+	/* mod->numvoices was used during loading to clamp md_sngchn.
+	   After loading it's used to remember how big mod->voice is.
+	*/
+	mod->numvoices = md_sngchn;
+
 	Player_Init_internal(mod);
 	return 0;
 }
@@ -2994,10 +3008,8 @@ void Player_Exit_internal(MODULE* mod)
 		pf=NULL;
 	}
 
-	if (mod->control)
-		MikMod_free(mod->control);
-	if (mod->voice)
-		MikMod_free(mod->voice);
+	MikMod_free(mod->control);
+	MikMod_free(mod->voice);
 	mod->control=NULL;
 	mod->voice=NULL;
 }
@@ -3012,8 +3024,10 @@ void Player_Exit(MODULE* mod)
 MIKMODAPI void Player_SetVolume(SWORD volume)
 {
 	MUTEX_LOCK(vars);
-	if (pf)
+	if (pf) {
 		pf->volume=(volume<0)?0:(volume>128)?128:volume;
+		pf->initvolume=pf->volume;
+	}
 	MUTEX_UNLOCK(vars);
 }
 
@@ -3087,7 +3101,7 @@ MIKMODAPI void Player_NextPosition(void)
 		pf->patbrk=0;
 		pf->vbtick=pf->sngspd;
 
-		for (t=0;t<pf->numchn;t++) {
+		for (t=0;t<NUMVOICES(pf);t++) {
 			Voice_Stop_internal(t);
 			pf->voice[t].main.i=NULL;
 			pf->voice[t].main.s=NULL;
@@ -3112,7 +3126,7 @@ MIKMODAPI void Player_PrevPosition(void)
 		pf->patbrk=0;
 		pf->vbtick=pf->sngspd;
 
-		for (t=0;t<pf->numchn;t++) {
+		for (t=0;t<NUMVOICES(pf);t++) {
 			Voice_Stop_internal(t);
 			pf->voice[t].main.i=NULL;
 			pf->voice[t].main.s=NULL;
@@ -3139,7 +3153,7 @@ MIKMODAPI void Player_SetPosition(UWORD pos)
 		pf->sngpos=pos;
 		pf->vbtick=pf->sngspd;
 
-		for (t=0;t<pf->numchn;t++) {
+		for (t=0;t<NUMVOICES(pf);t++) {
 			Voice_Stop_internal(t);
 			pf->voice[t].main.i=NULL;
 			pf->voice[t].main.s=NULL;
@@ -3149,12 +3163,12 @@ MIKMODAPI void Player_SetPosition(UWORD pos)
 			pf->control[t].main.s=NULL;
 		}
 		pf->forbid=0;
-		
+
 		if (!pos)
 			Player_Init_internal(pf);
 	}
 	MUTEX_UNLOCK(vars);
-}    
+}
 
 static void Player_Unmute_internal(SLONG arg1,va_list ap)
 {
@@ -3264,7 +3278,7 @@ static void Player_ToggleMute_internal(SLONG arg1,va_list ap)
 			}
 			break;
 		default:
-			if (arg1<pf->numchn) 
+			if (arg1<pf->numchn)
 				pf->control[arg1].muted=1-pf->control[arg1].muted;
 			break;
 		}
@@ -3311,7 +3325,7 @@ MIKMODAPI UWORD Player_GetChannelPeriod(UBYTE chan)
 	UWORD result=0;
 
 	MUTEX_LOCK(vars);
-    if (pf)
+	if (pf)
 	    result=(chan<pf->numchn)?pf->control[chan].main.period:0;
 	MUTEX_UNLOCK(vars);
 
@@ -3345,7 +3359,7 @@ MIKMODAPI void Player_TogglePause(void)
 MIKMODAPI void Player_SetSpeed(UWORD speed)
 {
 	MUTEX_LOCK(vars);
-	if (pf) 
+	if (pf)
 		pf->sngspd=speed?(speed<32?speed:32):1;
 	MUTEX_UNLOCK(vars);
 }
@@ -3384,18 +3398,17 @@ MIKMODAPI int Player_QueryVoices(UWORD numvoices, VOICEINFO *vinfo)
 	return numvoices;
 }
 
-
-// Get current module order
+/* Get current module order */
 MIKMODAPI int Player_GetOrder(void)
 {
 	int ret;
 	MUTEX_LOCK(vars);
-	ret = pf ? pf->sngpos :0; // pf->positions[pf->sngpos ? pf->sngpos-1 : 0]: 0;
+	ret = pf ? pf->sngpos :0; /* pf->positions[pf->sngpos ? pf->sngpos-1 : 0]: 0; */
 	MUTEX_UNLOCK(vars);
 	return ret;
 }
 
-// Get current module row
+/* Get current module row */
 MIKMODAPI int Player_GetRow(void)
 {
 	int ret;
@@ -3405,5 +3418,4 @@ MIKMODAPI int Player_GetRow(void)
 	return ret;
 }
 
-
 /* ex:set ts=4: */
